#!/bin/sh

cd `dirname $0`

die () {
  echo "*** regression.sh: $*" 1>&2
  exit 1
}

[ -x mcracetrack ] || die "could not find 'mcracetrack' program"

[ -x check-mcracetrack-solution ] || \
  die "could not find 'check-mcracetrack-solution' program"

[ -f makefile ] || die "could not find 'makefile'"

if [ x"`grep BACKWARD makefile`" = x ]
then
  BACKWARD=""
else
  BACKWARD="--only-backward --only-forward"
fi

if [ x"`grep COUNTPATHS makefile`" = x ]
then
  GC="-g"
else
  GC=""
fi

cd parcours

failed=0

run () {
  base=$1
  shift
  expected=$1
  shift
  [ -f $base ] || die "could not find parcours '$base'"
  echo -n $r mcracetrack $* $base 
  ../mcracetrack $* $base >$base.log 2>$base.err
  actual=$?
  echo -n " [$actual]"
  if [ ! $actual = $expected ]
  then
    echo -n "  !! BUT EXPECTED $expected !!"
    failed=`expr $failed + 1`
  else
    ../check-mcracetrack-solution $base $base.log >$base.clog 2>$base.cerr
    res=$?
    if [ $res != 0 ]
    then
      echo -n "  !! CHECK FAILED !!"
      failed=`expr $failed + 1`
    fi
  fi
  echo
  r=`expr $r + 1`
}
r=1

for opt in "" $BACKWARD
do
for gc in "" $GC
do

run starteqgoal 10 $opt $gc
run onestep0 10 -a 1 -i 1 $opt $gc
run onestep1 10 $opt $gc
run negstep1 10 $opt $gc
run onestep2 10 $opt $gc
run onestep3 10 $opt $gc
run twostep1 10 $opt $gc
run twostep2 10 -a 5 $opt $gc
run box0 10 -a 1 $opt $gc
run box1 10 -a 1 $opt $gc
run box2 10 -a 1 $opt $gc
run box3 10 -a 1 $opt $gc
run nostep1 20 -i 10 $opt $gc
run nostep1 20 -a 3 -i 3 $opt $gc
run nostep1 10 -a 2 -i 2 $opt $gc
run nostep1 10 -a 1 -i 1 $opt $gc
run maze1 20 -a 1 $opt $gc
run maze1 20 -a 2 $opt $gc
run points1 10 -a 1 $opt $gc
run maze2 10 $opt $gc
run maze3 10 -a 2 $opt $gc
run hseg1 20 -a 1 $opt $gc
run hseg2 20 -a 2 -i 2 $opt $gc
run maze4 10 -a 1 $opt $gc
run maze5 10 -a 1 $opt $gc
run maze6 10 -a 1 $opt $gc
run maze7 10 -a 1 $opt $gc

done
done

if [ $failed -gt 0 ]
then
  echo "$failed TESTS FAILED"
else
  echo "ALL TESTS SUCCEEDED"
fi
